#ifndef DISPATCHER_H
#define DISPATCHER_H

#include "msgval.h"

/**
 * @file Dispatcher.h
 * @brief Dispatches incoming socket events to worker threads
 */
/** 
 * @defgroup dispatcher Dispatcher

A QUICK EXPLANATION OF THESE FUNCTIONS:

The purpose of these functions is to use the best available event monitoring
mechanism available on the underlying platform to receive socket events, then
dispatch those socket events to a worker thread which then invokes a specified
handler function.  Every implementation will contain the main event loop for
the server, which will block indefinitely until an event is received.  Each
implementation is capable of receiving multiple event notifications within
a system call, including the three stateful mechanisms implemented here,
kqueues, /dev/poll, and epoll().  Due to the transactional nature of the
server, any of these implemenentations, with the exception of select(),
should be able to scale to thousands of active connections.

 * @{
 */


/** initialize  pipe, and dispatch thread */
void dispatcher_init(void);

/** signal dispatch thread to quit and free up internal data structures */
void dispatcher_destroy(void);

/** add another socket to listen to
 * @param sock the socket
 * @param event_handler the handler to call
 */
void dispatcher_add(int sock, void (*event_handler)(msgval *));

/** remove the socket
 * @param sock the socket to remove
 */
void dispatcher_remove(int sock);

/** @} */
#endif
