#ifndef EXECQUEUE_H
#define EXECQUEUE_H

#include "msgval.h"
/**
 * @file ExecQueue.h
 * @brief Lightweight interthread execution control queues
 */
/** 
 * @defgroup execqueue interthread execution control queues
 * @{
 */


/** ExecQueue data type */
typedef struct _ExecQueue *ExecQueue;

/**
 * Create a execution queue
 * @returns the queue
 */
ExecQueue execqueue_create();
/**
 * Destroy the execution queue
 * @param queue the queue to destroy
 */
void execqueue_destroy(ExecQueue queue);

/** writes a function invocation request to the given queue */
void execqueue_invoke(ExecQueue queue, void (*function)(msgval *), msgval *arg);

/**  terminates an execution loop.  It is equivalent to passing
   a NULL function pointer to execqueue_invoke() */
void execqueue_terminate(ExecQueue queue);

/** transfers control to an event listener and execution loop */ 
void execqueue_run(ExecQueue queue, void (*postfunc)());

/** @} */
#endif
