#ifndef HASHER_H
#define HASHER_H

/**
 * @file Hasher.h
 * @brief Abstract hash cipher interface
 */
/** 
 * @defgroup hasher Hasher
 * @{
 */


/** available algorithm types for the hash */
typedef enum {
	PDTP_HASH_MD5, /**< MD5 */
	PDTP_HASH_SHA1 /**< SHA1 */
} HashAlgorithm;

/** handle to the hasher */
typedef struct _Hasher *Hasher;

/** 
 * @brief create a hasher 
 * @param algorithm the algorithm to use
 * @returns the hasher
 */
Hasher hasher_create(HashAlgorithm algorithm);
/**
 * @brief destroy the hasher 
 */
void hasher_destroy(Hasher);

/** 
 * @brief return the digest size
 * @return the size
 */
unsigned hasher_digest_size(Hasher);
/**
 * @brief initalize the hasher
 */
void hasher_zero(Hasher);

/**
 * @brief add data to the hasher
 * @param buf the data to add
 * @param len size of the buffer
 */
void hasher_add_data(Hasher, const void *buf, unsigned len);

/**
 * @brief finish computing the hasher and return the digest
 * @param digest the digest returned
 */
 
void hasher_get_digest(Hasher, void *digest);

/** @} */
#endif
