/**
  @file Indexer.c
  @brief Manage a range of unique index keys
*/
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

#include <stdlib.h>
#include <assert.h>

#include "Indexer.h"
#include "xmalloc.h"

typedef struct _IndexRangeNode {
	unsigned start;
	unsigned end;
	struct _IndexRangeNode *link;
} *IndexRangeNode;

struct _Indexer {
	IndexRangeNode root;
};

Indexer indexer_create(unsigned min, unsigned max)
{
	Indexer im;
	
	assert(max >= min);

	im = NEW(Indexer);
	im->root = NEW(IndexRangeNode);
	im->root->start = min;
	im->root->end = max;
	im->root->link = 0;

	return im;
}

void indexer_destroy(Indexer im)
{
	IndexRangeNode t;
	
	while(im->root != 0) {
		t = im->root;
		im->root = im->root->link;

		free(t);
	}

	free(im);
}

int indexer_get(Indexer im, unsigned *index)
{
	IndexRangeNode t;

	if(im->root == 0)
		return -1;

	*index = im->root->start++;

	if(im->root->start > im->root->end) {
		t = im->root;
		im->root = im->root->link;
		free(t);
	}

	return 0;
}

/* XXX This is probably a bit more complicated than it needs to be */
void indexer_return(Indexer im, unsigned index)
{
	IndexRangeNode c, t;

	if(im->root == 0) {
		im->root = NEW(IndexRangeNode);
		im->root->start = index;
		im->root->end = index;
		im->root->link = 0;

		return;
	} 
	
	if(index < im->root->start - 1) {
		t = im->root;

		im->root = NEW(IndexRangeNode);
		im->root->start = index;
		im->root->end = index;
		im->root->link = t;

		if(t == 0)
			return;

		c = im->root;
	} else if(index == im->root->start - 1) {
		im->root->start--;

		return;
	} else if(im->root->link == 0) {
		if(index > im->root->end + 1) {
			im->root->link = NEW(IndexRangeNode);
			im->root->link->start = index;
			im->root->link->end = index;
			im->root->link->link = 0;
		} else if(index == im->root->end + 1)
			im->root->end++;

		return;
	} else {
		for(c = im->root; c->link != 0 && index > c->link->end + 1; c = c->link);

		if(c->link == 0) {
			c->link = NEW(IndexRangeNode);
			c->link->start = index;
			c->link->end = index;
			c->link->link = 0;

			return;
		}
		
		if(index == c->link->end + 1) {
			c = c->link;
			c->end++;
		} else if(index == c->link->start - 1) 
			c->link->start--;
		else if(index < c->link->start - 1 && index > c->end + 1) {
			t = c->link;

			c->link = NEW(IndexRangeNode);
			c->link->start = index;
			c->link->end = index;
			c->link->link = t;
		}
	}

	if(c->link != 0 && c->end == c->link->start - 1) {
		t = c->link;

		c->end = c->link->end;
		c->link = c->link->link;

		free(t);
	}
}
