/**
  @file Listener.c 
  @brief Functions for managing listener socket(s)
 */
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

#include <string.h>

#include "Config.h"
#include "Dispatcher.h"
#include "log.h"
#include "socketops.h"
#include "util.h"
#include "xmalloc.h"

static uint32_t listen_address = 0; 
static uint16_t listen_port = 0;
static Sock listen_sock = -1;

void listener_init(void (*connection_handler)(msgval *), unsigned default_port)
{
        char *address, *t;
        uint32_t new_listen_address;
        uint16_t new_listen_port;

        if((address = config_value("global", "listen_address"))) {
		if(!(t = strchr(address, ':')))
			new_listen_port = default_port;
		else {
			*t++ = '\0';
			new_listen_port = atoi(t);
		}
		
                new_listen_address = inet_addr(address);
                xfree(address);
        } else {
		log_printf("*** global::listen_address unset, defaulting to 0.0.0.0");
                new_listen_address = INADDR_ANY;
		new_listen_port = default_port;
	}

        if(new_listen_address != listen_address || new_listen_port != listen_port) {
                if(listen_sock != -1) {
                        dispatcher_remove(listen_sock);
                        socket_close(listen_sock);
                }

                listen_address = new_listen_address;
                listen_port = new_listen_port;

                if((listen_sock = listener_ipv4_tcp_create(listen_address, listen_port)) < 0) {
                        log_printf("!!! Warning: Couldn't bind listener socket to %s:%d", t = xinet_ntoa(listen_address), listen_port);
                        xfree(t);

                        /* Win32 version doesn't support config rehashing yet */
#ifdef WIN32
                        fatal("!!! Couldn't create listener socket!");
#endif

                        log_printf("!!! Please correct and rehash your server configuration");

                        listen_sock = -1;
                        listen_address = 0;
                        listen_port = -1;
                } else {
                        log_printf("*** Listening on %s:%d", t = xinet_ntoa(listen_address), listen_port);
                        xfree(t);

                        dispatcher_add(listen_sock, connection_handler);
                }
        }
}

void listener_destroy()
{
	if(listen_sock != -1) {
		dispatcher_remove(listen_sock);
		socket_close(listen_sock);
	}
}
