#ifndef LOCK_H
#define LOCK_H

/**
 * @file Lock.h
 * @brief Abstract thread synchronization locks
 */

/**
 * @defgroup lock Lock
 * @{
 */
 /** type of lock to create */
typedef enum { LOCK_EXCL,	/**< Fully exclusive lock */
               LOCK_SHARED	/**< Exclusive write, shared read */
} LockType;

/** Lock Data Type */
typedef struct _Lock *Lock;

/** Create the Lock */
Lock lock_create(LockType);
/** Destroy the lock */
void lock_destroy(Lock);

/** acquire a read only version of the lock, blocking if required */
void lock_reader_acquire(Lock);
/** release the read only version of the lock */
void lock_reader_release(Lock);

/** acquire a read/write version of the lock, blocking if required */
void lock_writer_acquire(Lock);
/** release the read/write version of the lock */
void lock_writer_release(Lock);

/** @} */
#endif /* LOCK_H */
