#ifndef SHA1_H
#define SHA1_H
/**
 * @file SHA1.h
 * @brief NIST Secure Hash Algorithm 
 */

/** 
 * @defgroup SHA1 SHA1 Secure hash algorithm
 * @ingroup hasher
 * @{ 
 */
/** SHA1 data type */
typedef struct _SHA1 *SHA1;

/** create the SHA1 structure */
SHA1 sha1_create();
/** destroy the SHA1 structure */
void sha1_destroy(SHA1);
/**
 * Start SHA1 accumulation.  
 */ 
void sha1_zero(SHA1);

/**
 * Update context to reflect the concatenation of another buffer full
 * of bytes.
 */
void sha1_add_data(SHA1, void *buf, unsigned len);

/**
 * Get the actual digest
 */
void sha1_get_digest(SHA1, unsigned char digest[20]);

/** @} */
#endif
