#ifndef TRANSACTION_H
#define TRANSACTION_H

#include <machdep.h>

/**
 * @file Transaction.h
 * @brief Code for constructing and processing transactions
 */

/**
 * @defgroup transaction Transaction processing
 * @{
 */
 
typedef struct _TransactionIn *TransactionIn;
typedef struct _TransactionOut *TransactionOut;

typedef enum {
	TXN_COMMAND,
	TXN_REPLY
} TransactionType;

TransactionIn transaction_read(int sock);
void transaction_in_destroy(TransactionIn);
TransactionType transaction_type(TransactionIn);
uint16_t transaction_id(TransactionIn);
uint32_t transaction_serial(TransactionIn);
int transaction_object_count(TransactionIn);
int transaction_object_index(TransactionIn, uint16_t type);
uint16_t transaction_object_id(TransactionIn, unsigned objno);
int transaction_object_length(TransactionIn, unsigned objno);
void *transaction_object_data(TransactionIn, unsigned objno);
int transaction_object_uint16(TransactionIn, unsigned objno, uint16_t *object);
int transaction_object_uint32(TransactionIn, unsigned objno, uint32_t *object);
char *transaction_object_string(TransactionIn, unsigned objno);
int transaction_object_unpack(TransactionIn, unsigned objno, const char *format, ...);
	
TransactionOut transaction_create(TransactionType, uint16_t tid, uint16_t obj_count);
TransactionOut transaction_copy(TransactionIn);
void transaction_set_serial(TransactionOut, uint32_t serial);
void transaction_out_destroy(TransactionOut);
void transaction_add_object(TransactionOut, uint16_t id, const void *data, uint32_t length);
void transaction_add_int16(TransactionOut, uint16_t id, uint16_t value);
void transaction_add_int32(TransactionOut, uint16_t id, uint32_t value);
void transaction_add_int64(TransactionOut, uint16_t id, uint64_t value);
void transaction_add_string(TransactionOut, uint16_t id, const char *string);
int transaction_object_pack(TransactionOut, uint16_t id, const char *format, ...);
int transaction_write(int sock, TransactionOut);

/** @} */
#endif /* TRANSACTION_H */
