/**
  @file Worker.c 
  @brief Worker thread implementation
*/
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <pthread.h>

#include "Worker.h"
#include "WorkerManager.h"
#include "ExecQueue.h"
#include "xmalloc.h"

struct _Worker {
	pthread_t tid;
	ExecQueue queue;
};

static pthread_key_t worker_key;
static pthread_once_t worker_once = PTHREAD_ONCE_INIT;

static void worker_once_init()
{
	pthread_key_create(&worker_key, 0);
}

static void worker_complete()
{
	workermgr_return_worker((Worker)pthread_getspecific(worker_key));
}

static void *worker_main(void *arg)
{
	Worker self = (Worker)arg;

	pthread_once(&worker_once, worker_once_init);
	pthread_setspecific(worker_key, self);

	execqueue_run(self->queue, worker_complete);

	pthread_exit(0);

	/* Unreached, but to suppress warnings on some sub-par compilers */
	return 0;
}

Worker worker_create()
{
	Worker worker;
	
	worker = NEW(Worker);
	worker->queue = execqueue_create();
	
	if(pthread_create(&worker->tid, 0, worker_main, worker) != 0) {
		execqueue_destroy(worker->queue);
		xfree(worker);

		return 0;
	}
	
	return worker;
}

void worker_destroy(Worker worker)
{
	execqueue_terminate(worker->queue);
	pthread_join(worker->tid, 0);

	execqueue_destroy(worker->queue);
	xfree(worker);
}

void worker_exec(Worker worker, void (*function)(msgval *), msgval *arg)
{
	execqueue_invoke(worker->queue, function, arg);
}
