/**
   @file WorkerManager.h 
   @brief Regulates and maintains thread pool
*/
/*
   Copyright (C)2002-03 Anthony Arcieri 
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.

 * The name of Anthony Arcieri may not be used to endorse or promote 
 products derived from this software without specific prior written 
 permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT 
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WORKERMANAGER_H
#define WORKERMANAGER_H

/**
 * @defgroup workermanaegr Worker Manager
 * @{
 */
#include "Worker.h"
#include "ExecQueue.h"
#include "msgval.h"

/* Limits for the thread pool */
#define DEFAULT_MAX_ACTIVE_THREADS      128
#define DEFAULT_MAX_INACTIVE_THREADS    8
#define DEFAULT_INITIAL_THREADS         1

void workermgr_init(void);
void workermgr_destroy(void);

/** Execute the given function on a worker thread */
void workermgr_exec(void (*function)(msgval *), msgval *arg);

/* These are called internally by workers */
void workermgr_return_worker(Worker t);
void workermgr_handle_crash(void);

/** @} */
#endif
