/**
  @file auth.c 
  @brief Server side authorization
*/
/*
   Copyright (C)2003-04 Anthony Arcieri
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

#include <string.h>

#include "HMAC.h"
#include "Hasher.h"
#include "RNG.h"
#include "auth.h"
#include "socketops.h"

/** Protocol magic numbers */
#define AUTH_MAGIC      0x41555448

/** XXX The cryptographic security of this function is questionable.  It should
   be audited by a cryptographic specialist prior to production use */
static int auth_hmac_sha1(int sock, const char *key)
{
	uint8_t randdata[20], client_digest[20], hmac_digest[20];

	/* Generate and send a 160-bit challenge message */
	rng_randbuf(randdata, 20);
	writesock(sock, randdata, 20);

	/* Read and validate the client's response */
	if(readsock(sock, client_digest, 20) < 0)
		return -1;


	hmac_compute(PDTP_HASH_SHA1, key, strlen(key), randdata, 20, hmac_digest);

	if(memcmp(client_digest, hmac_digest, 20))
		return 0;

	return 1;
}

int auth_passphrase(int sock, const char *passphrase)
{
	int r;
	const uint8_t ACK = 1, NACK = 0;
	uint16_t method;

	writesock_int32(sock, AUTH_MAGIC);

	for(;;) {
		if(readsock_int16(sock, &method) < 0)
			return -1;

		switch(method) {
			case AUTH_HMAC_SHA1:
				writesock(sock, &ACK, 1);

				if((r = auth_hmac_sha1(sock, passphrase)) < 0)
					return -1;

				if(r) {
					writesock(sock, &ACK, 1);
					return 0;
				}

				break;
		}

		writesock(sock, &NACK, 1);
	}
}
