/**
  @file fileops.c 
  @brief Platform independent file operations
 */
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
 */

#include <machdep.h>

#ifndef WIN32
#include <sys/stat.h>
#endif

#include "fileops.h"
#include "xmalloc.h"

struct _FileHandle {
#ifdef WIN32
	HANDLE handle;
#else
	int fd;
#endif /* WIN32 */
};

FileHandle file_open(const char *path)
{
	FileHandle file;

#ifdef WIN32
	HANDLE fh;

	if((fh = CreateFile(path, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, 0)) == INVALID_HANDLE_VALUE)
		return 0;

	file = NEW(FileHandle);
	file->handle = fh;
#else
	int fd;

	if((fd = open(path, O_RDWR)) < 0)
		return 0;

	file = NEW(FileHandle);
	file->fd = fd;
#endif /* WIN32 */

	return file;
}

void file_close(FileHandle file)
{
#ifdef WIN32
	CloseHandle(file->handle);
#else
	close(file->fd);
#endif /* WIN32 */

	xfree(file);
}

/* Read nbytes into the given buffer from the specified offset */
int file_read(FileHandle file, void *buffer, unsigned nbytes, uint64_t offset)
{
#ifdef WIN32
	return -1;	
#else

#ifdef HAVE_PREAD
	return pread(file->fd, buffer, nbytes, offset);
#else
#error Sorry, no support for hosts without pread() yet
#endif /* HAVE_PREAD */

#endif /* WIN32 */
}

int file_length(FileHandle file, uint64_t *length)
{
#ifdef WIN32
	ULARGE_INTEGER fsize;

	if((fsize.LowPart = GetFileSize(file, &fsize.HighPart)) == INVALID_FILE_SIZE)
		return -1;

	*length = fsize.QuadPart;
#else
	struct stat st;

	if(fstat(file->fd, &st) < 0)
		return -1;

	*length = st.st_size;
#endif /* WIN32 */

	return 0;
}
