#ifndef SOCKETOPS_H
#define SOCKETOPS_H

#include <machdep.h>
/** 
 * @file socketops.h 
 * @brief Platform independent socket functions
 */
 /**  
  * @defgroup socketopts Socket functions
  * @{
  */
#ifdef WIN32
typedef SOCKET Sock;
#else
typedef int Sock;
#endif

typedef struct _Addr *Addr;
typedef enum { SOCKET_BLOCK, SOCKET_NONBLOCK } BlockingType;

Addr addr_create_from_ipv4addr(uint32_t addr, uint16_t port);
Addr addr_create_from_hostname(const char *hostname, unsigned port);
void addr_destroy(Addr);

uint32_t addr_get_ipv4(Addr);
uint16_t addr_get_port(Addr);

Sock sock_nbconnect_to_addr(Addr);
int sock_nbconnect_complete(Sock);
Sock sock_connect_to_addr(Addr);

int set_blocking_mode(Sock, BlockingType);

Sock listener_ipv4_tcp_create(uint32_t source_addr, uint16_t port);
Sock listener_ipv4_tcp_accept(Sock, uint32_t *addr);
Sock listener_ipv4_udp_create(uint32_t source_addr, uint16_t port);
int listener_ipv4_udp_receive(Sock, void *buffer, unsigned length, uint32_t *addr);

#ifdef WIN32
#define socket_close(sock)	closesocket(sock)
#else
#define socket_close(sock)	close(sock)
#endif

ssize_t readsock_atomic(Sock, void *buf, size_t nbytes);
ssize_t writesock_atomic(Sock, void *buf, size_t nbytes);
	
int readsock(Sock, void *buf, size_t nbytes);
int writesock(Sock, const void *buf, size_t nbytes);

int readsock_int16(Sock, uint16_t *value);
int readsock_int32(Sock, uint32_t *value);

int writesock_int16(Sock, const uint16_t value);
int writesock_int32(Sock, const uint32_t value);

/** @} */
#endif /* SOCKETOPS_H */
