#include <CuTest.h>

#include "../Config.h"
#include "../xmalloc.h"

static const char *config_keys[] = {
	":thisiscorrect",
	"ints:a,b,c,d,e",
	"truth:foo,baz,biff1,biff2,biff3,biff4",
	"text:a,b",
	"list:a,b,c,d",
	0
};

void test_config(CuTest *tc)
{
	char *s;

	CuAssert(tc, "couldn't read configuration file",
		config_init("test.conf", config_keys) == 0);

	/* empty section header allowed */
	s = config_value("", "thisiscorrect");
	CuAssertStrEquals(tc, "really", s);
	xfree(s);

	/* integer values */
	CuAssertIntEquals(tc, 0, config_int_value("ints", "a")); /* atoi("a") => 0 */
	CuAssertIntEquals(tc, 0, config_int_value("ints", "b"));
	CuAssertIntEquals(tc, -34, config_int_value("ints", "c"));
	CuAssertIntEquals(tc, 16, config_int_value("ints", "d"));
	CuAssertIntEquals(tc, -1, config_int_value("ints", "e")); /* missing */

	/* truth values */
	CuAssertIntEquals(tc, 0, config_truth_value("truth", "foo"));
	CuAssertIntEquals(tc, 1, config_truth_value("truth", "baz"));
	CuAssertIntEquals(tc, 1, config_truth_value("truth", "biff1"));
	CuAssertIntEquals(tc, 1, config_truth_value("truth", "biff2"));
	CuAssertIntEquals(tc, 1, config_truth_value("truth", "biff3"));
	CuAssertIntEquals(tc, -1, config_truth_value("truth", "biff4")); /* missing */

	/* text */
	CuAssertStrEquals(tc, "some", config_value("text", "a"));
	CuAssertStrEquals(tc, "some text", config_value("text", "b"));

	/* list of values */
	CuAssertStrEquals(tc, "1", config_vlist_value("list", "a", 0));
	CuAssertStrEquals(tc, "2", config_vlist_value("list", "a", 1));
	CuAssertStrEquals(tc, "3", config_vlist_value("list", "a", 2));
	CuAssertStrEquals(tc, "4", config_vlist_value("list", "a", 3));

	CuAssertStrEquals(tc, "a", config_vlist_value("list", "b", 0));
	CuAssertStrEquals(tc, "b", config_vlist_value("list", "b", 1));
	CuAssertStrEquals(tc, "c", config_vlist_value("list", "b", 2));

	CuAssertStrEquals(tc, "a", config_vlist_value("list", "c", 0));
	CuAssertStrEquals(tc, "b", config_vlist_value("list", "c", 1));
	CuAssertPtrEquals(tc, NULL, config_vlist_value("list", "c", 2));

	CuAssertStrEquals(tc, "a,", config_vlist_value("list", "d", 0));
	CuAssertStrEquals(tc, "b", config_vlist_value("list", "d", 1));

	CuAssertPtrEquals(tc, NULL, config_vlist_value("list", "e", 0));

	config_destroy();
}
