#ifndef THREADOPS_H
#define THREADOPS_H

#include <machdep.h>
/**
 * @file threadops.h
 * @brief Portable thread operations
 */
/**
 * @defgroup threadops Thread operations
 * @{
 */
#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif /* WIN32 */

#ifdef WIN32
typedef HANDLE Thread;
typedef LPCRITICAL_SECTION Mutex;
typedef HANDLE Cond;
#endif

#ifdef HAVE_PTHREADS
typedef pthread_t *Thread;
typedef pthread_mutex_t *Mutex;
typedef pthread_cond_t *Cond;
#endif /* WIN32 */

Thread thr_create(void *(*thread_func)(void *arg), void *arg);
void thr_destroy(Thread);

#ifdef WIN32
#define thr_exit(arg)	ExitThread((DWORD)arg)
#endif /* WIN32 */

#ifdef HAVE_PTHREADS
#define thr_exit(arg)	pthread_exit(arg)
#endif /* HAVE_PTHREADS */
	
Mutex thr_mutex_create(void);
void thr_mutex_destroy(Mutex);

Cond thr_cond_create(void);
void thr_cond_destroy(Cond);

#ifdef WIN32
#define thr_mutex_lock(mutex)		EnterCriticalSection(mutex)
#define thr_mutex_unlock(mutex)		LeaveCriticalSection(mutex)

int thr_cond_wait(Cond, Mutex);
#define thr_cond_signal(cond)		PulseEvent(cond)
#endif /* WIN32 */

#ifdef HAVE_PTHREADS
#define thr_mutex_lock(mutex)		pthread_mutex_lock(mutex)
#define thr_mutex_unlock(mutex)		pthread_mutex_unlock(mutex)

#define thr_cond_wait(cond, mutex)	pthread_cond_wait(cond, mutex)
#define thr_cond_signal(cond)		pthread_cond_signal(cond)
#endif /* HAVE_PTHREADS */

/** @} */
#endif /* THREADOPS_H */
