/**
  @file transaction_replies.c
*/
/*
   Copyright (C)2001-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <stdarg.h>

#include "Transaction.h"
#include "txid.h"
#include "debug.h"
#include "xmalloc.h"

void reply_success(int sock, TransactionIn txn)
{
	TransactionOut reply;

	reply = transaction_create(TXN_REPLY, transaction_id(txn), 0);
	transaction_set_serial(reply, transaction_serial(txn));
	transaction_write(sock, reply);
	transaction_out_destroy(reply);
}

void reply_error(int sock, uint32_t serial, const char *format, ...)
{
	va_list ap;
	char *replystr;
	TransactionOut reply;

	reply = transaction_create(TXN_REPLY, TXN_ERROR, 1);
	transaction_set_serial(reply, serial);

	va_start(ap, format);
	replystr = xvasprintf(format, ap);
	va_end(ap);

#ifndef NDEBUG
	debug("Responding with error: %s", replystr);
#endif
	
	transaction_add_string(reply, OBJ_ERRMSG, replystr);
	free(replystr);

	transaction_write(sock, reply);
	transaction_out_destroy(reply);
}
