#ifndef TXID_H
#define TXID_H

/**
 * @file txid.h
 */

/**
 * @defgroup txid Transactions
 */
/****************
 * TRANSACTIONS *
 ****************/

/**
 * @defgroup txidError Error Type
 * @ingroup txid
 * @{
 */
#define TXN_ERROR       0x0
/** @} */
/**
 * @defgroup txidFile File Download transactions
 * @ingroup txid
 * @{
 */

#define TXN_DOWNLOAD	0x1
#define TXN_CANCELDL	0x2
#define TXN_GETCERT	0x3
#define TXN_SETPORT	0x4
#define TXN_SENDPIECE	0x10
#define TXN_RECVPIECE	0x11
#define TXN_AUTHUL	0x12
#define TXN_AUTHDL	0x13
#define TXN_XFERSUCCESS	0x14
#define TXN_XFERFAIL	0x15
#define TXN_XFERTERM	0x16

/** @} */
/**
 * @defgroup txidDirectory Directory listing
 * @ingroup txid
 * @{
 */


#define TXN_STATDIR     0x100
#define TXN_STATFILE	0x101
#define TXN_DIRLIST	0x102
#define TXN_FILEINFO    0x103
#define TXN_RICHDLIST	0x104
#define TXN_RICHFINFO	0x105

/** @} */
/**
 * @defgroup txidSrcFile Source File information
 * @ingroup txid
 * @{
 */

#define TXN_ADDHANDLE	0x200
#define TXN_REMHANDLE	0x201

/** @} */
/***********
 * OBJECTS *
 ***********/
/** 
 * @defgroup txidobjects Objects
 * @ingroup txid
 * @{
 */
#define OBJ_HANDLE	0x1
#define OBJ_PATH	0x2
#define OBJ_FILELEN	0x3
#define OBJ_PIECELEN	0x4
#define OBJ_PIECEID     0x5
#define OBJ_PIECELIST	0x6

/** @} */
/**
 * @defgroup objNet Network Entities
 * @ingroup txidobjects
 * @{
 */
/* Network entities */
#define OBJ_IPV4ADDR	0x10
#define OBJ_IPV6ADDR	0x11
#define OBJ_PORT	0x12

/** @} */
/**
 * @defgroup objHash Hashing algorithms
 * @ingroup txidobjects
 * @{
*/

#define OBJ_MD5HASH     0x20
#define OBJ_SHA1HASH	0x21
/** @} */

/**
 * @defgroup objcrypt Cryptographic signature checking 
 * @ingroup txidobjects
 * @{
 */

#define OBJ_X509CERT	0x40
#define OBJ_SIGNATURE	0x41

/** @} */
/**
 * @defgroup objdir directory objects 
 * @ingroup txidobjects
 * @{
 */

#define OBJ_SUBDIR      0x100
#define OBJ_FILE	0x101
#define OBJ_TIMESTAMP	0x102
#define OBJ_RDLDATA	0x103
#define OBJ_SUBDIR      0x100

/** @} */

/**
 * @defgroup objerr Error Objects
 * @ingroup txidobjects
 * @{
 */

#define OBJ_ERRMSG	0x500

/** @} */
#endif /* TXID_H */
