#!/usr/bin/perl

sub scansource {
my $basedir = shift;

open(MAKEFILE, "$basedir/Makefile") or die("Couldn't open $basedir/Makefile");
$makefile_data = join("", <MAKEFILE>);
close MAKEFILE;

$makefile_data =~ s/\\\n//g;
@makefile_values = split("\n", $makefile_data);

foreach $line (@makefile_values) {
	if($line =~ m/^OBJS=/) {
		$line =~ s/^.*=\s//;
		$line =~ s/\s/ /;
		$line =~ s/\.o/\.c/g;
		@source = split(" ", $line);
	}
}

foreach $line (@source) {
	$line =~ s/^/$basedir\//;
}

return @source;
}

sub genheader {
my $basedir = shift;
my $outfile = "$basedir/config_keys.h";
my @files = (@_, scansource($basedir));
my %sections;
my $allowed_names = "";

print("*** Creating $outfile from " . join(", ", @files) . "\n");

foreach $sourcefile (@files) {
        open(SRCIN, $sourcefile);

        while (<SRCIN>) {
                if (/config\w*_value\s*\(\s*"(.+)",\s*"(.+)"\s*\)/) {
                        print("Found key $1::$2\n");
                        
                        if ($sections{$1}) {
                                $sections{$1} .= ",$2";
                        } else {
                                $sections{$1} = $2;
                        }
                }
        }

        close SRCIN;
}	

foreach $section (keys(%sections)) {
        $allowed_names .= "\t\"$section:$sections{$section}\",\n";
}

open(CONFIG_KEYS, ">$outfile");

print CONFIG_KEYS <<END_OUT
/**
 * \@file  config_keys.h 
 * \@brief List of sections and keys for those sections used by
 * the configuration file.  
 *
 * Automatically generated by make-configkeys.pl.
 */

#ifndef CONFIG_KEYS_H
#define CONFIG_KEYS_H

static const char *config_keys[] = {
$allowed_names	0
};

#endif /* CONFIG_KEYS_H */
END_OUT
}

@commonfiles = scansource("common");

genheader("squall", @commonfiles);
genheader("squallhub", @commonfiles);
genheader("squallproxy", @commonfiles);
genheader("squirt", @commonfiles);
