/*
 @file ClientManager.c 
 @brief Central structure for storing all client metadata
*/
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

#include <assert.h>

/* Common headers */
#include <Collection.h>
#include <Lock.h>
#include <debug.h>
#include <log.h>
#include <util.h>
#include <xmalloc.h>

/* Local headers */
#include "Client.h"
#include "ClientManager.h"

static Collection client_table = 0;
static Lock client_table_lock = 0;

#define INITIALIZED	(client_table && client_table_lock)

void clientmgr_init()
{
	assert(!INITIALIZED);
	
	client_table_lock = lock_create(LOCK_SHARED);
	client_table = collection_create();
}

void clientmgr_reinit()
{
	assert(INITIALIZED);
}

void clientmgr_destroy()
{
	assert(INITIALIZED);
	
	lock_destroy(client_table_lock);
	client_table_lock = 0;
	
	collection_destroy(client_table, (void (*)(void *))client_destroy);
	client_table = 0;
}

int clientmgr_add_ipv4_connection(int sock, uint32_t addr)
{
	Client client;
	char *t;

	assert(INITIALIZED);

	log_printf("*** %s has requested the PDTP service", t = xinet_ntoa(addr));
	xfree(t);

	client = client_create(sock, addr);

	lock_writer_acquire(client_table_lock);
	collection_insert(client_table, sock, client);
	lock_writer_release(client_table_lock);

	return 0;
}

void clientmgr_remove_connection(int sock)
{
	Client client;
	char *t;

	assert(INITIALIZED);

	lock_writer_acquire(client_table_lock);
	client = (Client)collection_remove(client_table, sock);
	lock_writer_release(client_table_lock);

	/* Check to see internal structures are consistent */
	assert(client != 0);

	log_printf("--- %s disconnected from the PDTP service", t = xinet_ntoa(client_addr(client)));
	xfree(t);

	client_destroy(client);
}
