#ifndef PIECESET_H
#define PIECESET_H

/**
 * @file PieceSet.h
 * @brief Bit vector piece set management.
 */
typedef struct _PieceSet *PieceSet;

/** Creates an empty piece set that can represent pieces from 0 to size. */
PieceSet pieceset_create(int size);

/** Destroys a piece set. */
void pieceset_destroy(PieceSet set);

/** Returns a positive number if the piece is in the set, 0 if it is not. */
int pieceset_get_piece(PieceSet set, int piece);

/** Adds a piece to the set. */
void pieceset_add_piece(PieceSet set, int piece);

/** Adds a range of pieces to the set. */
void pieceset_add_range(PieceSet set, int start, int count);

/** Removes a piece from the set. */
void pieceset_remove_piece(PieceSet set, int piece);

/** Returns the intersection of set1 and set2. */
PieceSet pieceset_and(PieceSet set1, PieceSet set2);

/** Returns the intersection of the given sets. */
PieceSet pieceset_andv(PieceSet *sets, int numsets);

/** Returns the union of set1 and set2. */
PieceSet pieceset_or(PieceSet set1, PieceSet set2);

/** Returns the union of the given sets. */
PieceSet pieceset_orv(PieceSet *sets, int numsets);

/** Returns the set of all pieces not in the given set. */
PieceSet pieceset_not(PieceSet set);

/** Returns a random piece in the set. */
int pieceset_random_piece(PieceSet set);

#endif
