/**
  @file hubtxn.c 
  @brief Squall Server's handlers for hub to server transactions
 */
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
 */

#include <machdep.h>

/* Common headers */
#include <Collection.h>
#include <Transaction.h>
#include <debug.h>
#include <txid.h>
#include <xmalloc.h>

/* Local headers */
#include "TransferManager.h"
#include "hubtxn.h"

static int txn_addhandle(int sock, TransactionIn txn);
static int txn_remhandle(int sock, TransactionIn txn);
	
struct th_list_element {
        uint16_t txn_id;
        int (*handler)(int sock, TransactionIn);
};

static struct th_list_element transaction_list[] = { 
	{ TXN_ADDHANDLE, txn_addhandle },
	{ TXN_REMHANDLE, txn_remhandle },
	{ 0, 0 }
};

static Collection transaction_table = 0;

void hubtxn_init()
{
        int i;

        transaction_table = collection_create();

        for(i = 0; transaction_list[i].txn_id; i++)
                collection_insert(transaction_table, (CollectionKey)transaction_list[i].txn_id, (void *)transaction_list[i].handler); 
}

void hubtxn_destroy()
{
        collection_destroy(transaction_table, 0);
}

int hubtxn_exec(int sock, TransactionIn txn)
{ 
        int ret = 0; 
        int (*handler)(int, TransactionIn);

        if((handler = (int (*)(int, TransactionIn))collection_lookup(transaction_table, (CollectionKey)transaction_id(txn)))) 
		ret = handler(sock, txn);
	
        transaction_in_destroy(txn);

        return ret;
}

static int txn_addhandle(int sock, TransactionIn txn)
{
	int i;
	uint32_t handle, piece_count;
	char *path;
	
#ifndef NDEBUG
	debug("Got addhandle");
#endif

	if((i = transaction_object_index(txn, OBJ_HANDLE)) < 0)
		return 0;

	if(transaction_object_uint32(txn, i, &handle) < 0)
		return 0;

	if((i = transaction_object_index(txn, OBJ_PATH)) < 0)
		return 0;

	path = transaction_object_string(txn, i);

	if((i = transaction_object_index(txn, OBJ_PIECEID)) < 0)
		goto done;

	if(transaction_object_uint32(txn, i, &piece_count) < 0)
		goto done;

	xfermgr_add_file(handle, path, piece_count);
	
done:
	xfree(path);
	return 0;
}

static int txn_remhandle(int sock, TransactionIn txn)
{
	int i;
	uint32_t handle;

#ifndef NDEBUG
	debug("Got remhandle");
#endif
	
	if((i = transaction_object_index(txn, OBJ_HANDLE)) < 0)
		return 0;

	if(transaction_object_uint32(txn, i, &handle) < 0)
		return 0;

	xfermgr_remove_file(handle);

	return 0;
}
