/**
  @file netevent.c 
  @brief Network event handlers (invoked by dispatcher)
*/
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

/* Common headers */
#include <Dispatcher.h>
#include <Transaction.h>
#include <log.h>
#include <msgval.h>
#include <socketops.h>
#include <util.h>
#include <xmalloc.h>

/* Local headers */
#include "ClientManager.h"
#include "clienttxn.h"
#include "netevent.h"

/* Protocol magic numbers */
#define PDTP_MAGIC	0x50445450
#define PDR4_MAGIC	0x50445234
#define PTPP_MAGIC	0x50745050
#define AUTH_MAGIC	0x41555448

static int pdtp_connection_handler(int sock, uint32_t addr)
{
	uint16_t value;
		
	/* XXX Peer redirection support goes here */
	writesock_int32(sock, PDTP_MAGIC);

	/* Minimum protocol version supported, currently hardcoded to 1 */
	writesock_int16(sock, 1);

	/* Maximum protocol version supported, currently hardcoded to 1 */
	writesock_int16(sock, 1);

	/* Read the client's selected protocol version */
	if(readsock_int16(sock, &value) < 0)
		return -1;

	if(value != 1)
		return -1;

	/* Add the connection to the client manager */
	clientmgr_add_ipv4_connection(sock, addr);

	/* Add the connection back to the dispatcher */
	dispatcher_add(sock, client_transaction_handler);

	return 0;
}

/** Handler for incoming connections */
void incoming_connection_handler(msgval *arg)
{
	Sock sock;
	uint32_t addr, value;
	char *t;

	sock = listener_ipv4_tcp_accept(arg->num, &addr);
	dispatcher_add(arg->num, incoming_connection_handler);

	if(sock < 0)
		return;

	log_printf("+++ Connection from %s", t = xinet_ntoa(addr)); 
	xfree(t);

	/* Read the magic number from the client */
	if(readsock_int32(sock, &value) < 0)
		goto err;

	if(value == PDTP_MAGIC) {
		if(pdtp_connection_handler(sock, addr) < 0)
			goto err;

		return;
	}
	
	return;
err: 
	log_printf("--- %s disconnected", t = xinet_ntoa(addr)); 
	xfree(t);
					
	socket_close(sock);
}

void client_transaction_handler(msgval *arg)
{
	TransactionIn txn;
	int sock = arg->num;

	if(!(txn = transaction_read(sock)) || clienttxn_exec(sock, txn) < 0) {
		clientmgr_remove_connection(sock);
		return;
	}

	dispatcher_add(sock, client_transaction_handler);
}
