/**
  @file DirList.c 
  @brief Abstract directory listing interface for VFS monitor
*/
/*
   Copyright (C)2003-04 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

/* System headers */
#include <string.h>

/* Common headers */
#include <DirEntry.h>
#include <xmalloc.h>

/* Local headers */
#include "DirList.h"

typedef struct _DirListNode {
	struct DirEntry entry;
	struct _DirListNode *next;
} *DirListNode;

struct _DirList {
	DirListNode head, tail, cur;
	int count;
};

DirList dirlist_create(void)
{
	DirList dirlist = NEW(DirList);

	dirlist->head = dirlist->tail = dirlist->cur = 0;
	dirlist->count = 0;

	return dirlist;
}

void dirlist_destroy(DirList dirlist)
{
	DirListNode temp;

	while((temp = dirlist->head)) {
		dirlist->head = temp->next;
		xfree(temp);
	}

	xfree(dirlist);
}

void dirlist_add_entry(DirList dirlist, struct DirEntry *entry)
{
	DirListNode node;
	
	if(!dirlist->head)
		node = dirlist->head = dirlist->tail = dirlist->cur = NEW(DirListNode);
	else {
		node = dirlist->tail->next = NEW(DirListNode);
		dirlist->tail = node;
	}

	memcpy(&node->entry, entry, sizeof(struct DirEntry));
	node->next = 0;

	dirlist->count++;
}

int dirlist_get_entry(DirList dirlist, struct DirEntry *entry)
{
	if(!dirlist->cur)
		return -1;

	memcpy(entry, &dirlist->cur->entry, sizeof(struct DirEntry));
	dirlist->cur = dirlist->cur->next;

	return 0;
}

void dirlist_rewind(DirList dirlist)
{
	dirlist->cur = dirlist->head;
}

int dirlist_count(DirList dirlist)
{
	return dirlist->count;
}
