/**
  @file FileEntry.c 
  @brief File information container object for file manager
 */
/*
   Copyright (C)2004 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
*/

#include <machdep.h>

/* Common headers */
#include <DirEntry.h>
#include <xmalloc.h>

/* Local headers */
#include "FileEntry.h"
#include "FileHashes.h"

struct _FileEntry {
	char *path;
	uint32_t handle;
	unsigned ts_year;
	unsigned ts_ysec;
	uint64_t length;

	FileHashes hashes;
};

FileEntry fileentry_create(const char *path, uint32_t handle)
{
	FileEntry file;
	struct DirEntry entry;

	if(direntry_get(path, &entry) < 0)
		return 0;

	file = NEW(FileEntry);

	file->path = xstrdup(path);
	file->handle = handle;
	file->ts_year = entry.ts_year;
	file->ts_ysec = entry.ts_ysec;
	file->length = entry.length;

	file->hashes = 0;

	return file;
}

FileEntry fileentry_copy(FileEntry file)
{
	FileEntry ret = NEW(FileEntry);

	ret->path = xstrdup(file->path);
	ret->handle = file->handle;
	ret->ts_year = file->ts_year;
	ret->ts_ysec = file->ts_ysec;
	ret->length = file->length;

	if(file->hashes)
		ret->hashes = filehashes_copy(file->hashes);
	else
		ret->hashes = 0;

	return ret;
}

void fileentry_destroy(FileEntry file)
{
	xfree(file->path);

	if(file->hashes)
		filehashes_destroy(file->hashes);

	xfree(file);
}

/* XXX The interface of this function probably needs to change */
char *fileentry_get_path(FileEntry file)
{
	return file->path;
}

uint32_t fileentry_get_handle(FileEntry file)
{
	return file->handle;
}

uint64_t fileentry_get_length(FileEntry file)
{
	return file->length;
}

FileHashes fileentry_get_hashes(FileEntry file)
{
	if(!file->hashes)
		return 0;

	return filehashes_copy(file->hashes);
}

void fileentry_set_hashes(FileEntry file, FileHashes hashes)
{
	if(file->hashes)
		filehashes_destroy(file->hashes);

	file->hashes = filehashes_copy(hashes);
}
