/**
  @file FileHasher.c
  @brief Generates list of MD5/SHA1 for pieces within a file
 */
/* 
   Copyright (C)2004 Anthony Arcieri 
   All rights reserved.

   For licensing information, please see the file LICENSE 
 */

#include <machdep.h>

#include <string.h>

/* Common headers */
#include <Hasher.h>
#include <txid.h>
#include <xmalloc.h>

/* Local headers */
#include "FileHashes.h"

struct _FileHashes {
	HashAlgorithm cipher;
	unsigned length;
	uint32_t piece_length, piece_count;
	void *buffer;
};

FileHashes filehashes_create(HashAlgorithm cipher, unsigned piece_length, void *hashlist, unsigned length)
{
	FileHashes hashes = NEW(FileHashes);

	hashes->cipher = cipher;
	hashes->length = length;
	hashes->piece_length = piece_length;

	hashes->piece_count = (unsigned)(length / piece_length);

	if(length % piece_length)
		hashes->piece_count++;

	hashes->buffer = xmalloc(length);
	memcpy(hashes->buffer, hashlist, length);

	return hashes;
}

FileHashes filehashes_copy(FileHashes hashes)
{
	FileHashes ret = NEW(FileHashes);

	ret->cipher = hashes->cipher;
	ret->length = hashes->length;
        ret->piece_length = hashes->piece_length;
	ret->piece_count = hashes->piece_count;

	ret->buffer = xmalloc(hashes->length);
	memcpy(ret->buffer, hashes->buffer, hashes->length);

	return ret;
}

void filehashes_destroy(FileHashes hashes)
{
	xfree(hashes->buffer);
	xfree(hashes);
}

uint32_t filehashes_get_piece_length(FileHashes hashes)
{
	return hashes->piece_length;
}

uint32_t filehashes_get_piece_count(FileHashes hashes)
{
	return hashes->piece_count;
}

void filehashes_add_to_transaction(TransactionOut txn, FileHashes hashes)
{
	/* Initialize to zero to suppress warnings */
	uint16_t obj_type = 0;

	switch(hashes->cipher) {
		case PDTP_HASH_MD5:
			obj_type = OBJ_MD5HASH;
			break;
		case PDTP_HASH_SHA1:
			obj_type = OBJ_SHA1HASH;
			break;
	}

	transaction_add_object(txn, obj_type, hashes->buffer, hashes->length);
}
