#ifndef FILEMANAGER_H
#define FILEMANAGER_H

#include <machdep.h>

/* Common headers */
#include <Lock.h>

/* Local headers */
#include "FileEntry.h"
#include "FileHashes.h"

/** 
 * @file FileManager.h
 * @brief Manages files being served
 */
void filemgr_init();
void filemgr_reinit();
void filemgr_destroy();

/* Make the given file available for download through the file manager  */
void filemgr_add_file(const char *path);

/* Remove the given file from the file manager */
void filemgr_remove_file(const char *path);

/* Set a particular file's hash list */
int filemgr_set_hashes(const char *path, FileHashes hashes);

/* Fetch (a copy) of an entry by path.  Free with filemgr_return_entry() */
FileEntry filemgr_get_entry_by_path(const char *path, LockType);

/* Fetch (a copy) of an entry bh handle.  Free with filemgr_return_entry() */
FileEntry filemgr_get_entry_by_handle(uint32_t handle, LockType);

/* Return an entry to the file manager (currently free the copy) */
void filemgr_return_entry(FileEntry);

/* Send a series of add handle transactions for all currently exported files */
void filemgr_blast_handles(int sock);

#endif
