#include "../VFSMonitor.h"
#include "../config_keys.h"

#include <Config.h>
#include <xmalloc.h>

#include <CuTest.h>

#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#  include <unistd.h>
#else /* WIN32 */
#endif /* WIN32 */

static int vfsmonitor_initialized = 0;

#define ASSERT_INIT CuAssert(tc, "vfs monitor not initialized", \
	vfsmonitor_initialized != 0);

/* Helper Functions */

static void delete_file(const char *path)
{
	if(unlink(path) == -1)
		printf("unlink(%s) failed\n", path);
}

static void make_directory(const char *path)
{
}

/* Callbacks */

void vfs_scan_complete(const char *path)
{
	printf("scan complete in %s\n", path);
}

void vfs_lose_file(const char *path)
{
	printf("lost file %s\n", path);
}

void vfs_find_file(const char *path)
{
	printf("found file %s\n", path);
}

/* Test Cases */

void test_init(CuTest *tc)
{
	unsigned files, dirs;

	config_init("../pdtpss.conf.dist", config_keys);

	CuAssert(tc, "The VFS Monitor failed to initialize",
		vfsmonitor_init("vfsmon") == 0);

	vfsmonitor_initialized = 1;

	CuAssert(tc, "vfsmonitor_stats() error return code",
		vfsmonitor_stats(&files, &dirs) == 0);
	CuAssertIntEquals(tc, 0, files);
	CuAssertIntEquals(tc, 1, dirs);
}

void test_translate_path(CuTest *tc)
{
#ifdef WIN32
	static const char *expected = "vfsmon\foo";
#else
	static const char *expected = "vfsmon/foo";
#endif
	char *path;

	ASSERT_INIT

	path = vfsmonitor_translate_path("foo");
	CuAssertPtrEquals_Msg(tc, "vfsmonitor_translate_path() accepts paths "
		"without a leading /", 0, path);

	path = vfsmonitor_translate_path("/foo");
	CuAssertStrEquals(tc, expected, path);
	xfree(path);
}

void test_file_create(CuTest *tc)
{
	FILE *fp;

	ASSERT_INIT
puts("creating file");
	CuAssert(tc, "couldn't create testfile",
		(fp = fopen("vfsmon/testfile", "w")) != 0);

	fclose(fp);
puts("closed file");
	delete_file("vfsmon/testfile");
puts("deleted file");
}

void test_destroy(CuTest *tc)
{
	ASSERT_INIT

	vfsmonitor_destroy();
}

static CuSuite *vfsmonitor_suite(void)
{
	CuSuite *suite = CuSuiteNew();

	SUITE_ADD_TEST(suite, test_init); /* must be first */
	SUITE_ADD_TEST(suite, test_translate_path);
	SUITE_ADD_TEST(suite, test_file_create);
	SUITE_ADD_TEST(suite, test_destroy); /* must be last */

	return suite;
}

int main(int argc, char **argv)
{
	CuString *output;
	CuSuite *suite;

	suite = vfsmonitor_suite();
	CuSuiteRun(suite);

	output = CuStringNew();
	CuSuiteSummary(suite, output);
	CuSuiteDetails(suite, output);
	puts(output->buffer);

	return 0;
}
