#!/bin/sh

# Auto generate single AllTests file for CuTest.
# Searches through all *.c files in the current directory.
# Prints to stdout.
# Author: Asim Jalis
# Date: 01/08/2003

# Change the first line to run the script with /bin/sh so that bash
# need not be installed. Seems to work fine on Linux and FreeBSD.
# Added prototype for printf() at the top and a return 0; in main()
# to the output so GCC would be happier.  - Matthew Bishop

if test $# -eq 0 ; then FILES=*.c ; else FILES=$* ; fi

echo '/* This is auto-generated code. Edit at your own peril. */

#include "CuTest.h"

extern int printf(const char *format, ...);
'

cat $FILES | grep -i '^void Test' | 
    sed -e 's/(.*$//' \
        -e 's/$/(CuTest*);/' \
        -e 's/^/extern /'

echo \
'

void RunAllTests(void) 
{
    CuString *output = CuStringNew();
    CuSuite* suite = CuSuiteNew();

'
cat $FILES | grep -i '^void Test' | 
    sed -e 's/^void //' \
        -e 's/(.*$//' \
        -e 's/^/    SUITE_ADD_TEST(suite, /' \
        -e 's/$/);/'

echo \
'
    CuSuiteRun(suite);
    CuSuiteSummary(suite, output);
    CuSuiteDetails(suite, output);
    printf("%s\n", output->buffer);
}

int main(void)
{
    RunAllTests();
    return 0;
}
'
